#include <stdio.h>
#include "image.h"

static int getValue(Image img, int x, int y)
{
	int v = pointg(img, x, y);

	if(v < 64)
		return 3;
	else if(v < 160)
		return 2;
	else if(v < 224)
		return 1;
	else
		return 0;
}

static unsigned char getByte(Image img, int x, int y, int plane)
{
	int mask = (plane ? 1: 2);

	return
	(getValue(img, x, y + 0) & mask ? 0x01: 0) |
	(getValue(img, x, y + 1) & mask ? 0x02: 0) |
	(getValue(img, x, y + 2) & mask ? 0x04: 0) |
	(getValue(img, x, y + 3) & mask ? 0x08: 0) |
	(getValue(img, x, y + 4) & mask ? 0x10: 0) |
	(getValue(img, x, y + 5) & mask ? 0x20: 0) |
	(getValue(img, x, y + 6) & mask ? 0x40: 0) |
	(getValue(img, x, y + 7) & mask ? 0x80: 0);
}

void mkpat(Image img)
{
	int x, y;

	for(y = 0; y < img.height; y += 8) {
		printf("\t.db ");
		for(x = 0; x < img.width - 1; x++)
			printf("0x%02x, ", getByte(img, x, y, 0));
		printf("0x%02x\n", getByte(img, x, y, 0));
	}
}

unsigned char *mkmap(unsigned char *buf, char *file, int planes)
{
	Image img = IMG_NULL;
	int plane, x, y;
	unsigned char *p = buf;

	imgLoad(&img, file);

	for(plane = 0; plane < planes; plane++)
		for(y = 0; y < img.height; y += 8)
			for(x = 0; x < img.width; x++)
				*p++ = getByte(img, x, y, plane);

	imgFree(&img);
	return p;
}

int getlen1(unsigned char *buf, int n)
{
	unsigned char *p;

	for(p = buf + 1; p < buf + n && *p != *(p - 1) && p < buf + 256; p++)
		;
	return (int )(p - buf);
}

int getlen2(unsigned char *buf, int n)
{
	unsigned char *p;

	for(p = buf; p < buf + n && p < buf + 256 && *p == *buf; p++)
		;

	return (int )(p - buf);
}

void pack(unsigned char *buf, int n)
{
	int len;
	unsigned char *p = buf;

	while(n > 0) {
		/* sA̒𓾂 */
		len = getlen1(p, n);

		/* sA */
		if (len > 0) {
			printf("\t.db %d\n", len % 256);
			for(; len > 0; len--, n--)
				printf("\t.db 0x%02x\n", *p++);
		}

		/* A̒𓾂 */
		len = getlen2(p, n);

		/* A */
		if (len > 0) {
			printf("\t.db %d\n", len % 256);
			p += len;
			n -= len;
		}
	}
}

int main(int argc, char *argv[])
{
	Image img = IMG_NULL;
	int i;
	static unsigned char buf[2048], *p;

	if(argc == 3) {
		printf("%s::\n", argv[2]);
		imgLoad(&img, argv[1]);
		mkpat(img);
	} else {
		printf("%s::\n", argv[1]);
		p = buf;
		*p++ = atoi(argv[2]); /* goal_rl */
		*p++ = atoi(argv[3]); /* goal_horiz */
		*p++ = atoi(argv[4]) * 4; /* goal_x */
		*p++ = atoi(argv[6]) * 4 + 4; /* goal_y(HIGH) */
		*p++ = atoi(argv[5]) * 4; /* goal_y(LOW) */
		*p++ = atoi(argv[7]); /* lap_start */
		*p++ = atoi(argv[8]); /* kart_pos_start(X) */
		*p++ = atoi(argv[9]); /* kart_pos_start(Y) */
		*p++ = atoi(argv[10]); /* kart_dir_start */
		p = mkmap(p, argv[11], 1);
		p = mkmap(p, argv[12], 2);
		pack(buf, (int )(p - buf));
		printf("\t.dw 59999\n");
		printf("\t.dw 59999\n");
	}
	return 0;
}

/* eof */
